import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

// simple demonstration program: reads a grayscale image, inverts its pixel
// values, creates a new image to hold the result, and write out the new image using
// the color map of the original (src) image.
//
//	java JavaShowImages data\A1_Y83_2D.gif

public class JavaShowImages {

	public static BufferedImage srcIm, destIm;
	static JFrame f;
	static JPanel imgPan; 

	public JavaShowImages()
	{
	 }

	public JavaShowImages(String str)
	{ processInput(str);
	 }


    		// returns the grayscale values of the image pixels
    		// in row-major format, one byte per pixel
    public static byte[] getBytes(BufferedImage bim) {
	WritableRaster wr = bim.getWritableTile(0, 0);
	byte[] b = (byte[])(wr.getDataElements(0, 0, bim.getWidth(), bim.getHeight(), null));
	bim.releaseWritableTile(0, 0);
	return b;
    }

    public static void main(String[] args) {
	if(args.length != 1) {
	    System.err.println("usage: JavaShowImages [filename]");
	    System.exit(1);
	}
	processInput(args[0]);
    }	// END::main()


	// Process the input:: called form main with file-name for the source image
	public static void processInput(String str)
	{
	   try {
		 // open the image
	    	srcIm = ImageIO.read(new File(str));   // javax.imageio.ImageIO

	    	// be sure we're working in 8-bit grayscale
	    	if(srcIm.getType() != BufferedImage.TYPE_BYTE_GRAY) {
			System.err.println("\n\t Image not grayscale!!!");
	    	}

	    	// get the image pixels as bytes
	    	byte[] srcB = getArrayFromImage(srcIm);
		byte[] destB;

	    	// make a new array of inverted pixels
	    	// ONLY USE IF YOU WANT TO INVERT IMAGE INTENSITIES FROM THIS JAVA PROGRAM
		// - In general this is left to the CPP routines, instead,
		destB = filterImage(srcB);

	    	// create the new image
	    	destIm = makeImageFromArray(destB);

	    	// save/write the result
	    	ImageIO.write(destIm, "pgm", new File("output.pgm"));

	    	showBothImages (srcIm,destIm, 
			new String("Filter within Java code::JavaShowImages.java:: Input <======> Output"));
	   } catch(IOException ie) { ie.printStackTrace();   }
    }	// END::processInput()


	// Show input and output (filtered) images together in a JFrame
	public static void showBothImages (BufferedImage srcIm, BufferedImage destIm)
	{	showBothImages (srcIm, destIm, 
			new String ("************ Java_CPP_JNI_Interface_Demo ***************"));
	 }

	public static void showBothImages (BufferedImage srcIm, BufferedImage destIm, String str)
	{
	    // now display the image
	    f = new JFrame(str);
	    imgPan = new JPanel();
	    imgPan.setLayout(new GridLayout(1, 2));
	    imgPan.add(new ImagePanel(srcIm));
	    imgPan.add(new ImagePanel(destIm));
	    f.getContentPane().setLayout(new java.awt.GridLayout(1, 1));
	    f.getContentPane().add(new JScrollPane(imgPan));
	    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    f.pack();
	    f.setVisible(true);
     }	// END::showBothImages()

		// Whatever the image filter is (this should change for various applications
	public static byte[] filterImage(byte[] srcB)
	{	// make a new array of inverted pixels
	    	byte[] destB = new byte[srcB.length];
	    	for(int i = 0; i < srcB.length; i++)
			destB[i] = (byte)(srcB[i] ^ 0xff);
		return destB;
	 }

		// creates a new image using the dimensions of the (default) sourceImage
	public static BufferedImage makeImageFromArray(byte[] destB)
	{	// create the new image
	    	BufferedImage destImg = makeImage(destB, srcIm.getWidth(), srcIm.getHeight());
		return destImg;
	 }

    		// creates a new image of the given width and height from the
    		// given array of bytes
    	public static BufferedImage makeImage(byte[] bytes, int width, int height) {
		BufferedImage bim=new BufferedImage(width, height, BufferedImage.TYPE_BYTE_GRAY);
		WritableRaster wr = bim.getWritableTile(0, 0);
		wr.setDataElements(0, 0, width, height, bytes);

			// recreate the output (dest, filtered) using the input (src) image color map
		bim = new BufferedImage(srcIm.getColorModel(), wr, true, null);
		bim.releaseWritableTile(0, 0);
		return bim;
    	}

	public static byte[] getArrayFromImage(BufferedImage srcImg)
	{	// create the new image
	    	byte[] array = getBytes(srcImg);
		return array;
	 }

	// Additional Classes:**************************************************

    // 1. Image display class - wraps an image in a swing component
    static class ImagePanel extends JPanel {
	BufferedImage im;
	public ImagePanel(BufferedImage im) {
	    this.im = im;
	    setPreferredSize(new Dimension(im.getWidth(), im.getHeight()));
	}
	public void paint(Graphics g) { g.drawImage(im, 0, 0, this); }
    }

}
